------------------------------------------------------
-- IPM. Image Processing Module example
-- Description: Calculates the negative image
-- University of Alicante. Depto. TIC
-- Author: Sergio Cuenca
-- Date: 13/7/99
-------------------------------------------------------

-------------------------------------------------------
----  DATAPATH
-------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity datapath is
    port (clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        add_en: in STD_LOGIC;
        pix_rg: in STD_LOGIC;
        pix: in STD_LOGIC_VECTOR (7 downto 0);
        addr: out STD_LOGIC_VECTOR (14 downto 0);
        last: out STD_LOGIC;
        ppix: out STD_LOGIC_VECTOR (7 downto 0)
    );
end datapath;

architecture datapath_arch of datapath is
signal cnt:std_logic_vector (14 downto 0);
signal d: std_logic_vector (7 downto 0);
begin

-- Address generator ----------------------------------------
ADDGEN: process(clock,reset)
begin
	-- reset asynchronous
		if reset='1' then
			cnt <= (others=>'0'); -- initial address bank 0
	-- address counter increments on rising edge of clock
		elsif (clock'event and clock='1') then
			if add_en='1' then
				cnt <= cnt + 1;
			end if;	
		end if;
end process;
addr<=cnt;
last<=cnt(14);

-- Pixel register -------------------------------------------
PIXREG: process(pix_rg,reset)
begin
	if reset='1' then
		d <= (others=>'0');
	elsif (pix_rg'event and pix_rg='1') then
		d <= pix;
	end if;
end process;

-- Restador ------------------------------------------------
ppix<="11111111"-d;

end datapath_arch;
-------------------------------------------------------
----  FSM
-------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;

use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

library SYNOPSYS;
use SYNOPSYS.attributes.all;

entity fsm is 
  port (clock: in STD_LOGIC;
        last: in STD_LOGIC;
        reset: in STD_LOGIC;
        start: in STD_LOGIC;
        add_en: out STD_LOGIC;
        add_oe: out STD_LOGIC;
        endproc: out STD_LOGIC;
        m_oe: out STD_LOGIC;
        m_we: out STD_LOGIC;
        pix_oe: out STD_LOGIC;
        pix_rg: out STD_LOGIC);
end;

architecture fsm_arch of fsm is


-- SYMBOLIC ENCODED state machine: Sreg0
type Sreg0_type is (DEC, ENDP, GBRST, INIT, RD, WR);
signal Sreg0: Sreg0_type;

begin
--concurrent signal assignments
--diagram ACTIONS;


Sreg0_machine: process (clock, reset)

begin

if reset='1' then
	Sreg0 <= GBRST;
elsif clock'event and clock = '1' then
	case Sreg0 is
		when DEC =>
			if last='1' then
				Sreg0 <= ENDP;
			elsif last='0' then
				Sreg0 <= INIT;
			end if;
		when ENDP =>
		when GBRST =>
			if start='0' then
				Sreg0 <= INIT;
			end if;
		when INIT =>
			Sreg0 <= RD;
		when RD =>
			Sreg0 <= WR;
		when WR =>
			Sreg0 <= DEC;
		when others =>
			null;
	end case;
end if;
end process;

-- signal assignment statements for combinatorial outputs
endproc_assignment:
endproc <= '1' when (Sreg0 = ENDP) else
           '0';

pix_oe_assignment:
pix_oe <= '1' when (Sreg0 = WR) else
          '0';

add_oe_assignment:
add_oe <= '1' when (Sreg0 = INIT) else
          '1' when (Sreg0 = RD) else
          '1' when (Sreg0 = WR) else
          '0';

m_we_assignment:
m_we <= '0' when (Sreg0 = WR) else
        '1';

m_oe_assignment:
m_oe <= '1' when (Sreg0 = WR) else
        '0';

pix_rg_assignment:
pix_rg <= '1' when (Sreg0 = RD) else
          '0';

add_en_assignment:
add_en <= '1' when (Sreg0 = WR) else
          '0';

end fsm_arch;

-------------------------------------------------------
----  TOP
-------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

entity ipm is
    port (
        clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        start: in STD_LOGIC;
	pix: in STD_LOGIC_VECTOR(7 downto 0);
        addr: out STD_LOGIC_VECTOR (14 downto 0);
        add_oe: out STD_LOGIC;
        ppix: out STD_LOGIC_VECTOR(7 downto 0);
        pix_oe: out STD_LOGIC;
        endproc:out STD_LOGIC;
        m_oe:out STD_LOGIC;
        m_we:out STD_LOGIC);
end ipm;

architecture ipm_arch of ipm is

	component datapath
	port(clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        add_en: in STD_LOGIC;
        pix_rg: in STD_LOGIC;
        pix: in STD_LOGIC_VECTOR (7 downto 0);
        addr: out STD_LOGIC_VECTOR (14 downto 0);
        last: out STD_LOGIC;
        ppix: out STD_LOGIC_VECTOR (7 downto 0)
        );
    	end component;
    	
    	component fsm
    	port (
        clock: in STD_LOGIC;
        reset: in STD_LOGIC;
        start: in STD_LOGIC;
        last: in STD_LOGIC;  
        endproc: out STD_LOGIC;
        pix_rg: out STD_LOGIC;
        add_en:out STD_LOGIC;
        m_oe: out STD_LOGIC;
        m_we: out STD_LOGIC;
        add_oe: out STD_LOGIC;
        pix_oe: out STD_LOGIC
	);
        end component;

        
signal adden_s, last_s,pixrg_s: std_logic;

begin
u0: datapath port map(clock=>clock,reset=>reset,add_en=>adden_s,pix_rg=>pixrg_s,
	pix=>pix,addr=>addr,last=>last_s,ppix=>ppix);

u1: fsm port map(clock=>clock, reset=>reset,start=>start,last=>last_s,endproc=>endproc,
	 pix_rg=>pixrg_s, add_en=>adden_s,m_oe=>m_oe, m_we=>m_we, add_oe=>add_oe, pix_oe=>pix_oe);		

end ipm_arch;

